{ *************************************************************************** }
{                                                                             }
{ Hotzonu 1.5ch Script Unit                                                   }
{                                                                             }
{ 2001-04 Hotzonu Project, Monazilla Software Foundation 2001                 }
{                                                                             }
{ *************************************************************************** }

unit Zonu1_5ch;

interface

uses
  Classes, SysUtils, ZonuClass, ZonuCommon, ZonuHttp, Hzbconst;

type
  PParamData  = ^TParamData;
  TParamData  = record
    Subject : string;
    InRes : boolean;
    TempData: string;
    Res : TRes;
  end;

  T1_5chScript = class(TBBS)
  private
    { private }
  protected
    { protected }
    function GetThreadListType: TThreadListAccessType; override;
    function GetThreadDownloadMode: TThreadDownloadMode; override;
    function GetNeedHRefLink: boolean; override;
    function GetNeedLinkTarget: boolean; override;
    function GetRelativeLink: boolean; override;
    function GetIsPublished: boolean; override;
    procedure ZonuDataThreadList(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndThreadList(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuThreadListError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
    procedure ZonuDataRes(Sender: TObject; Downloader: TDownloader;
      Data: string; TotalSize, DownloadSize: integer; var MoreData: boolean); override;
    procedure ZonuEndRes(Sender: TObject; Downloader: TDownloader;
      Error: boolean); override;
    procedure ZonuResError(Sender: TObject; Downloader: TDownloader;
      ErrorCode: integer; ErrorMsg, LastResponse: string); override;
  public
    { public }
    procedure ParseSubject(subject: string; out FileName: string;
      out Title: string; out Count: integer); override;
    procedure ParseDat(const ALine: string; const index: integer;
      var ARes: TRes; out ASubject: string); override;
    procedure DownloadThreadList(Downloader: TThreadListDownloader;
      const NoCache, ASync, InfSave: boolean); override;
    procedure DownloadThreadItem(Downloader: TResDownloader;
      const NoCache, ASync, InfSave, AutoPilot: boolean); override;
    procedure SendRes(Uploader: TResUploader); override;
    procedure SendThread(Uploader: TThreadUploader); override;
    function  GetDisplayUrl(ThreadItem: TThreadItem): string; override;
    function ConvUrl(const Url: string): string; override;
  end;

implementation

uses
  MonaUtils;

{ T1_5chScript }

//
//  Xbhꗗ̎擾@
//
function T1_5chScript.GetThreadListType: TThreadListAccessType;
begin
  Result  :=  tlLump;
end;

//
//  Xbh̎擾@
//
function T1_5chScript.GetThreadDownloadMode: TThreadDownloadMode;
begin
  Result  :=  dmText;
end;

//
//  http://ւ̃Nǉ
//
function T1_5chScript.GetNeedHRefLink: boolean;
begin
  Result  :=  True;
end;

//
//  NւTarget=_blankǉ
//
function T1_5chScript.GetNeedLinkTarget: boolean;
begin
  Result  :=  False;
end;

//
//  BBSEBU[hŕ\邩ǂ
//
function T1_5chScript.GetIsPublished: boolean;
begin
  Result  :=  True;
end;

//
//  Xbhꗗ_E[h
//
procedure T1_5chScript.DownloadThreadList(Downloader: TThreadListDownloader;
  const NoCache, ASync, InfSave: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataThreadList;
  FZonuHttp.OnZonuEnd       :=  ZonuEndThreadList;
  FZonuHttp.OnZonuError     :=  ZonuThreadListError;
  FZonuHttp.Url             :=  GetSubjectUrl(Downloader.Board);
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.GetModified(Downloader.Board.CachePath + 'subject.inf', ASync);
end;

//
//  Xbhꗗ擾
//
procedure T1_5chScript.ZonuDataThreadList(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize,
  DownloadSize: integer; var MoreData: boolean);
var
  dl        : TThreadListDownloader;
  AFileName : string;
  ATitle    : string;
  ACount    : integer;
  ThreadItem: TThreadItem;
begin
  inherited;
  if  (AnsiPos('<a href', Data) > 0)  then  begin
    dl  :=  TThreadListDownloader(Downloader);
    ParseSubject(Data, AFileName, ATitle, ACount);
    ThreadItem                    :=  TThreadItem.Create;
    ThreadItem.BBS                :=  Self;
    ThreadItem.Site               :=  dl.Site;
    ThreadItem.BoardID            :=  dl.Board.BoardID;
    ThreadItem.No                 :=  0;
    ThreadItem.DatFileName        :=  AFileName + '.dat';
    ThreadItem.Title              :=  ATitle;
    ThreadItem.AgeCount           :=  ACount;
    ThreadItem.RemoteDatFileName  :=  GetDatUrl(dl.Board, AFileName);
    ThreadItem.LocalDatFileName   :=  dl.Board.CachePath + AFileName + '.dat';
    dl.FireDataEvt(ThreadItem, DownloadSize, TotalSize, MoreData);
  end;
end;

//
//  Xbhꗗ擾I
//
procedure T1_5chScript.ZonuEndThreadList(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
begin
  Dispose(Downloader.ParamData);
  inherited;
end;

//
//  Xbhꗗ擾G[
//
procedure T1_5chScript.ZonuThreadListError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;

end;

//
//  Xbhꗗ̉
//
procedure T1_5chScript.ParseSubject(subject: string; out FileName,
  Title: string; out Count: integer);
begin
  inherited;
  Parse1_5Subject(subject, FileName, Title, Count);
end;

//
//  X̉
//
procedure T1_5chScript.ParseDat(const ALine: string;
  const index: integer; var ARes: TRes; out ASubject: string);
begin
  inherited;
  Parse2chDat(ALine, index, ARes, ASubject);
end;

//
//  \URL
//
function T1_5chScript.GetDisplayUrl(ThreadItem: TThreadItem): string;
begin
  Result  :=  ThreadItem.RemoteDatFileName;
end;

//
//  Xbh̃_E[h
//
procedure T1_5chScript.DownloadThreadItem(Downloader: TResDownloader;
  const NoCache, ASync, InfSave, AutoPilot: boolean);
var
  ParamData : PParamData;
  FZonuHttp : TZonuHttp;
begin
  New(ParamData);
  ParamData.Subject         :=  '';
  ParamData.InRes           :=  False;
  ParamData.TempData        :=  '';
  ParamData.Res             :=  nil;  
  Downloader.ParamData      :=  ParamData;
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Downloader      :=  Downloader;
  FZonuHttp.OnZonuData      :=  ZonuDataRes;
  FZonuHttp.OnZonuEnd       :=  ZonuEndRes;
  FZonuHttp.OnZonuError     :=  ZonuResError;
  FZonuHttp.Url             :=  Downloader.ThreadItem.RemoteDatFileName;
  FZonuHttp.NoCache         :=  NoCache;
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.NeedDataEvent   :=  True;
  FZonuHttp.CacheReady      :=  False;
  FZonuHttp.InfSave         :=  InfSave;
  FZonuHttp.Username        :=  Downloader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Downloader.Board.BasicAuthorization_Password; //8
  FZonuHttp.CacheReady      :=  Downloader.CacheReady;
  Downloader.HttpObj        :=  FZonuHttp;
  FZonuHttp.GetText(ChangeFileExt(Downloader.ThreadItem.LocalDatFileName, '.inf'), ASync);
end;

//
//  X擾
//
procedure T1_5chScript.ZonuDataRes(Sender: TObject;
  Downloader: TDownloader; Data: string; TotalSize, DownloadSize: integer;
  var MoreData: boolean);
var
  dl  : TResDownloader;
  ParamData : PParamData;
  ASubject : string;
  s : string;
begin
  dl  :=  TResDownloader(Downloader);
  ParamData :=  PParamData(dl.ParamData); 
  if  (AnsiPos('<span class="Name">', Data) > 0) and (not ParamData.InRes)  then
  begin
    ParamData.InRes           :=  True;
    ParamData.TempData        :=  '';
    ParamData.Res             :=  TRes.Create;
    ParamData.Res.ThreadTitle :=  ParamData.Subject;
    ParamData.Res.BBS         :=  Self;
    ParamData.Res.Site        :=  dl.Site;
    s :=  Copy(Data, AnsiPos('>', Data) + 1, Length(Data));
    s :=  Copy(s, 1, AnsiPos('</a>', s) - 1);
    ParamData.Res.No   :=  StrToIntDef(s, 0);
    ParamData.Res.Name :=  '';
    s   :=  Copy(Data, AnsiPos('<span class="Name">', Data) + 19, Length(Data));
    s   :=  Copy(s, 1, AnsiPos('<', s) - 1);
    ParamData.Res.Name  :=  ParamData.Res.Name + Trim(s);
    if  (AnsiPos('<span class="Trip">', Data) > 0)  then  begin
      s   :=  Copy(Data, AnsiPos('<span class="Trip">', Data) + 19, Length(Data));
      s   :=  Copy(s, 1, AnsiPos('<', s) - 1);
      ParamData.Res.Name  :=  ParamData.Res.Name + '</B>' + Trim(s) + '<B>';
    end;
  end else
  if  (AnsiPos('<span style="font-size: 80%;">', Data) > 0) and
      (AnsiPos('<dd>', Data) = 0) and
      (ParamData.InRes)  then  begin
    //t
    s   :=  Copy(Data, AnsiPos('[', Data) + 1, Length(Data));
    s   :=  Copy(s, 1, AnsiPos(']', s) - 1);
    ParamData.Res.Date  :=  TrimTag(s);
    if  (AnsiPos('(ID:', Data) > 0)  then  begin
      s   :=  Copy(Data, AnsiPos('(ID:', Data) + 4, Length(Data));
      s   :=  Copy(s, 1, AnsiPos(')</span>', s) - 1);
      ParamData.Res.ID :=  s;
    end;
  end else
  if  (AnsiPos(#9 + '<dd>', Data) = 1) and
      (AnsiPos('</dd>', Data) > 0) and (ParamData.InRes) then
  begin
    //{
    s :=  Copy(Data, AnsiPos('<dd>', Data) + 4, Length(Data));
    s :=  Copy(s, 1, AnsiPos('</dd>', s) - 1);
    if  (Copy(s, Length(s) - 11, 12) = '<br /><br />')  then  begin
      s :=  Copy(s, 1, Length(s) - 12);
    end;
    s :=  TrimHrefTag(s);
    ParamData.Res.Body  :=  StringReplace(s, '<br />', '<br>', [rfReplaceAll]);
  end else
  if  (AnsiPos(#9#9#9 + '<dd>', Data) = 1) and
      (AnsiPos('form.cgi?', Data) = 0) and
      (AnsiPos('</dd>', Data) > 0) and (ParamData.InRes) then
  begin
    //sX
    s :=  Copy(Data, AnsiPos('<dd>', Data) + 4, Length(Data));
    s :=  Copy(s, 1, AnsiPos('</dd>', s) - 1);
    if  (ParamData.TempData = '')  then  ParamData.Res.Body   :=  ParamData.Res.Body + '<BR><BR>';
    ParamData.Res.Body   :=  ParamData.Res.Body + '<DL><DT><DD>' +  s  + '</DL>';
    ParamData.TempData   :=  s;
  end else
  if  (AnsiPos('</form>', Data) = 1) and (ParamData.InRes) then  begin
    //̏I

    dl.FireDataEvt(ParamData.Res, DownloadSize, TotalSize);

    //FireResFound(Sender, Adapter, ParamData.Res, TotalSize, Size);
    //if (not FromCache)  then  TThreadItem(Adapter.SourceObject).DownloadCount := TThreadItem(Adapter.SourceObject).DownloadCount + 1;
    //if  (not FromCache) and (ParamData.Subject <> '')  then  begin
    //  TThreadItem(Adapter.SourceObject).Title :=  ParamData.Subject; //
    //end;
    ParamData.InRes  :=  False;
  end else
  if  (AnsiPos('<title>', Data) = 1) and (not ParamData.InRes)  then  begin
    ParamData.Subject  :=  TrimTag(Data);
    if  (dl.ThreadItem.Title = '')  then  begin
      if  (ASubject <> '')  then  begin
        dl.ThreadItem.Title :=  ASubject;
      end;
    end;
  end;
{
var
  dl  : TResDownloader;
  ASubject : string;
  Res : TRes;
begin
  dl  :=  TResDownloader(Downloader);
  Res :=  TRes.Create;
  ParseDat(Data, 0, Res, ASubject);
  Res.No          :=  dl.ResList.Count + 1;
  Res.BBS         :=  dl.BBS;
  Res.Site        :=  dl.Site;
  Res.ThreadTitle :=  ASubject;
  if  (dl.ThreadItem.Title = '')  then  begin
    if  (ASubject <> '')  then  begin
      dl.ThreadItem.Title :=  ASubject;
    end;
  end;
  dl.FireDataEvt(Res, DownloadSize, TotalSize);
}
end;

//
//  X擾I
//
procedure T1_5chScript.ZonuEndRes(Sender: TObject;
  Downloader: TDownloader; Error: boolean);
var
  ParamData:  PParamData;
begin
  ParamData :=  PParamData(Downloader.ParamData);
  Dispose(ParamData);
  inherited;
end;

//
//  X擾G[
//
procedure T1_5chScript.ZonuResError(Sender: TObject;
  Downloader: TDownloader; ErrorCode: integer; ErrorMsg, LastResponse: string);
begin
  inherited;

end;

//
//  
//
procedure T1_5chScript.SendRes(Uploader: TResUploader);
var
  FZonuHttp : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetSendUrl(Uploader.Board, Uploader.ThreadItem);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.BBS.GetReadCgiUrl(Uploader.ThreadItem);
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetSendParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetSendCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

//
//  VX
//
procedure T1_5chScript.SendThread(Uploader: TThreadUploader);
var
  FZonuHttp   : TZonuHttp;
begin
  FZonuHttp                 :=  TZonuHttp.Create(nil);
  FZonuHttp.Uploader        :=  Uploader;
  FZonuHttp.OnZonuSendEnd   :=  ZonuSendEnd;
  FZonuHttp.OnZonuSendError :=  ZonuSendError;
  FZonuHttp.Url             :=  GetNewThreadUrl(Uploader.Board);
  FZonuHttp.UserAgent       :=  USER_AGENT;
  FZonuHttp.SendReferer     :=  Uploader.Board.Url;
  FZonuHttp.SendContent     :=  Uploader.SendData.ConvSendContent(GetNewThreadParam);
  FZonuHttp.SendCookie      :=  Uploader.SendData.ConvSendContent(GetNewThreadCookie);
  FZonuHttp.Username        :=  Uploader.Board.BasicAuthorization_UserName; //8
  FZonuHttp.Password        :=  Uploader.Board.BasicAuthorization_Password; //8
  FZonuHttp.Send;
end;

function T1_5chScript.ConvUrl(const Url: string): string;
var
  BBS,Dat : string;
  sl: TStringList;
  s: string;
  i: integer;
begin
  sl  :=  TStringList.Create;
  s   :=  '';
  try
    for i :=  Length(Url) downto  1 do  begin
      if  (Url[i] = '/')  then  begin
        if  (s <> '') then  sl.Insert(0, s);
        s :=  '';
      end else begin
        s :=  Url[i] + s;
      end;
    end;

    //http://1.5ch.info/inf_prac/log_html/all_log10179287185.html
    if  (AnsiPos('/all_log', Url) > 0) then begin
      Dat :=  sl[sl.Count - 1];
      Dat :=  Copy(Dat, 8, Length(Dat));
      Dat :=  Copy(Dat, 1, AnsiPos('.', Dat) - 1);
      BBS :=  sl[sl.Count - 3];
      Result  :=  'http://' + ExtractHostName(Url) + 'cgi-bin/1ch/read.cgh?' +
                  'fol=' + bbs + '&log=' + Dat;
    end else
    //http://1.5ch.info/cgi-bin/1ch/read.cgi/inf_prac/10179287185/l50
    //http://1.5ch.info/cgi-bin/1ch/read.cgi/inf_prac/10179287185/2-5n
    if  (AnsiPos('/read.cgi/', Url) > 0) then begin
      for i :=  0 to  sl.Count - 1  do  begin
        if  (sl[0] =  'read.cgi') then  begin
          sl.Delete(0);
          Break;
        end else begin
          sl.Delete(0);
        end;
      end;
      BBS :=  sl[0];
      Dat :=  sl[1];
      {
      ex  :=  '';
      if  (sl.Count > 2)  then  begin
        s :=  sl[2];
        if  (s  <>  '') then  begin
          if  (s[1] = 'l')  then  begin
            //
          end else
          if  (AnsiPos('-', s) > 0) then  begin
            f   :=  StrToInt(StringReplace(ChooseString(s, '-', 0), 'n', '', [rfReplaceAll]));
            t   :=  StrToInt(StringReplace(ChooseString(s, '-', 1), 'n', '', [rfReplaceAll]));
            n   :=  (AnsiPos('n', s) > 0);
            ex  :=  '&s=' + IntToStr(f) + '&e=' + IntToStr(t) +
          end;
        end;
      end;
      }
      Result  :=  'http://' + ExtractHostName(Url) + 'cgi-bin/1ch/read.cgh?' +
                  'fol=' + bbs + '&log=' + Dat;
    end else begin
      Result  :=  Url;
    end;
  finally
    sl.Free;
  end;
  //http://1.5ch.info/cgi-bin/1ch/read.cgi?fol=inf_prac&log=10179287185
end;

function T1_5chScript.GetRelativeLink: boolean;
begin
  Result  :=  False;
end;

end.
